/*
  This app helps the parents to take care of their baby.
  The types of scenarios are:
  1. Click on the left button in the cpx in order to publish a post in Facebook in the Babysitter group.
   The user need to set the wanted hour by clicking on the right button, and then shake the cpx.
  2. Every morning at 8:00 oclock the LEDs are painted in red / orange / blue according to the weather.
  3. Every day between 7:00 until 22:00 every 240 minutes the cpx makes a sound and turns on lights to
   remind the parent that the baby needs to be fed.
  4. At night when the baby is crying the cpx plays a tune and paints the LEDs to calm the baby.
  5. When the room temperature is too low or too high, the cpx increses / decreses one degree from the temperature of the air conditioner,
   In addition it shine in red if it has raised the temperature anUd blue if it has lowered the temperature.

  We used irReceiver in order to change the degress of the air conditioner.
  We used speaker in order to play a melody when the baby crying and when it's time to feed the baby
  we used X, Y and Z motion sensors to check if the watch was shaked.
  We used microphone in order to check if the baby is caying.
  We used the left button in order to start count the time for the babysitter.
  We used the right button in order to determine the time for the babysitter.
  We used the temperature sensor in order to check if we need to change the degress of the air conditioner.

  The circuit:
  input: motion(X and Y and Z), left button, right button, ir, microphone, temperature.
  output: sound, leds.

  Created By:
  Inbal Arazi 205915432
  Almog Levi 316604776
*/

#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB

// blynk-cloud.com/eV3OtLz8QQlCB7xXc6izlz8l_vNPqIIF/update/V0?value=50

#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>
#include "adafruit_mini_codes.h"
#include "pitches.h"

char auth[] = "eV3OtLz8QQlCB7xXc6izlz8l_vNPqIIF";
char ssid[] = "deco";
char pass[] = "ofir2020limor";

#define EspSerial Serial1
#define ESP8266_BAUD 115200U
//Defines for a air conditioner
#define MY_PROTOCOL NECX
#define MY_BITS 32
#define UP 0x4FBC03F
#define DOWN 0x4FB827D


ESP8266 wifi(&EspSerial);

long lastreported = 0;
float sample;
float temp;
boolean isCrying = false;
boolean isParentAwake = false;
boolean lightOn = false;
boolean firstTime = true;
boolean checkTemp = true;
boolean nightMode = false;
const int NUM_OF_NOTES = 26;
const int ROLL_THRESHOLD = 30;
const int YELLOW = 108;
const int ORANGE = 30;
const int MINTEMP = 22;
const int MAXTEMP = 25;
const int SPEED = 2;
int babySitterTime = 0;


// Jingle Bells!
int melody[] = {
  NOTE_B4, NOTE_B4, NOTE_B4,
  NOTE_B4, NOTE_B4, NOTE_B4,
  NOTE_B4, NOTE_D5, NOTE_G4, NOTE_A4,
  NOTE_B4, 0,
  NOTE_C5, NOTE_C5, NOTE_C5, NOTE_C5,
  NOTE_C5, NOTE_B4, NOTE_B4, NOTE_B4, NOTE_B4,
  NOTE_D5, NOTE_D5, NOTE_C5, NOTE_A4,
  NOTE_G4
};

// Durations... 8 = 1/8, 4 = 1/4, 3 = ~3/8, 2 = 1/2, 1 = Full
int noteDurations[] = {
  4, 4, 2,
  4, 4, 2,
  4, 4, 3, 8,
  1, 4,
  4, 4, 3, 8,
  4, 4, 4, 8, 8,
  4, 4, 4, 4,
  1
};

void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
}

void loop() {
  Blynk.run();
  //Clear pixels in case that the user press on right button
  if (CircuitPlayground.rightButton() && lightOn)
  {
    CircuitPlayground.clearPixels();
    lightOn = false;
  }

  //Checks if the user pressed in the left button
  if (CircuitPlayground.leftButton())
  {
    //Set babysitter time
    setBabySitterTime();
  }

  if (checkTemp)
  {
    //Checks and change the air conditioner degrees if neccecery
    detectTemp();
  }

  //Checks if it's night
  //if (nightMode)
  //{
    if (millis() - lastreported > 1000L)
    {
      //Checks if the baby is caying
      detectSound();
      lastreported = millis();
    }
  //}
}

// Checks if the temperature in the room is between Min temp to Max temp. If not it decrease/increase the air-conditioner degrees.
void detectTemp()
{
  //Measures the room temperature
  temp = CircuitPlayground.temperature();

  // Checks whether the temperature is not in the desired range
  if (temp > MAXTEMP || temp < MINTEMP)
  {
    // Updates blynk that the air conditioner degrees have changed
    Blynk.virtualWrite(V6, 1);
    // Change the air conditioner degrees
    setTemp(temp > MAXTEMP);
    checkTemp = false;
  }
}

// Checks if the baby is crying. If this is the first cry in the last 5 minutes it plays a music,
// otherwise it updates blynk to wake the parent
void detectSound()
{
  if (!isParentAwake)
  {
    sample = CircuitPlayground.mic.soundPressureLevel(10);
    Serial.println(sample);
    if (sample >= 70)
    {
      Blynk.virtualWrite(V0, sample, firstTime);
      firstTime = !firstTime;
    }
  }
}

//Read the wanted time to babysitter from the user
void setBabySitterTime()
{
  int count = 0;
  while (!detectShake()) //checks if the user shaked the cpx
  {
    if (CircuitPlayground.rightButton()) //checks if the user press on the right button of the cpx
    {
      count = (count + 1) % 24; // add 1 to the counter
    }
  }

  Blynk.virtualWrite(V5, count); //send the time to blynk
}

// Decarese/increse the airconditioner according to the room temp
void setTemp(boolean decrease)
{
  if (decrease)
  {
    //press on down button
    CircuitPlayground.irSend.send(MY_PROTOCOL, DOWN, MY_BITS);
    shine(0, 0, 255);
  }
  else {
    //press on up button
    CircuitPlayground.irSend.send(MY_PROTOCOL, UP, MY_BITS);
    shine(255, 0, 0);
  }
}

//Checks if the user shaked the cpx
boolean detectShake()
{
  // Compute total acceleration
  double X = 0;
  double Y = 0;
  double Z = 0;

  for (int i = 0; i < 10; i++) {
    X += CircuitPlayground.motionX();
    Y += CircuitPlayground.motionY();
    Z += CircuitPlayground.motionZ();
    delay(1);
  }

  X /= 10;
  Y /= 10;
  Z /= 10;

  double totalAccel = sqrt(X * X + Y * Y + Z * Z);

  // Play sound if rolling
  if (totalAccel > ROLL_THRESHOLD) {
    return true;
  }

  return false;
}

// Illuminates the CPX lights
void shine(int r, int g, int b)
{
  CircuitPlayground.clearPixels();
  for (int i = 0; i < 10; i++)
  {
    CircuitPlayground.setPixelColor(i, r, g, b);
    delay(40);
    CircuitPlayground.setPixelColor(i, 0, 0, 0);
  }
}

// Plays music accompanied by lights to calm the baby
void playAlarm() {
  for (int thisNote = 0; thisNote < NUM_OF_NOTES; thisNote++) { // play notes of the melody
    if (melody[thisNote] != 0) {
      if (thisNote % 2) {
        for (int p = 0; p <= 8; p = p + 2) {
          CircuitPlayground.setPixelColor(p, 255, 255, YELLOW);
          CircuitPlayground.setPixelColor(p + 1, 255, 71, ORANGE);
        }
      }
      else {
        for (int p = 0; p <= 8; p = p + 2) {
          CircuitPlayground.setPixelColor(p, 255, 71, ORANGE);
          CircuitPlayground.setPixelColor(p + 1, 255, 255, YELLOW);
        }
      }
    }

    int noteDuration = (1000 / SPEED) / noteDurations[thisNote];
    CircuitPlayground.playTone(melody[thisNote], noteDuration);

    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
    for (int p = 0; p <= 9; p++) {
      CircuitPlayground.setPixelColor(p, 0, 0, 0);
    }
  }
}

BLYNK_WRITE(V1) {
  temp = param.asDouble(); // assigning incoming value from pin V1 to a variable temp
  lightOn = true;
  showTemp();
}

BLYNK_WRITE(V3) {
  //Play music
  playAlarm();
}

BLYNK_WRITE(V4) {
  int isSleepBit = param.asInt(); // assigning incoming value from pin V4 to a variable isSleepBit
  if (isSleepBit == 0) //checks if the parent is sleeping
  {
    isParentAwake = false; //change to parent awake mode
  }
  else {
    isParentAwake = true; //change to parent sleep mode
  }
}

BLYNK_WRITE(V7) {
  checkTemp = true; // assigning incoming value from pin V7 to a variable temp
}

BLYNK_WRITE(V2) {
  int colorPixels[] = {0x0000cc , 0x008000, 0x4dc3ff, 0x00b350, 0xffff0, 0xff0000, 0xe62e00, 0xff33bb, 0x99ffff, 0x0000ff};
  for (int j = 0; j < 10; j++)
  {
    CircuitPlayground.setPixelColor(j, colorPixels[j]);
    delay(50);
  }

  CircuitPlayground.playTone(NOTE_E4, 200);
  CircuitPlayground.playTone(NOTE_GS3, 100);
  CircuitPlayground.playTone(NOTE_C4, 200);
  CircuitPlayground.playTone(NOTE_A4 , 300);

  for (int k = 1; k <= 10; k++)
  {
    for (int j = 0; j < 10; j++)
    {
      CircuitPlayground.setPixelColor(j, max((colorPixels[j] - k * (colorPixels[j] / 10)), 0));
    }

    delay(100);
  }
}

//set color pixels according to the temperature
void showTemp() {
  long color;
  if (temp < 17)
  {
    color = 0x0033cc; // blue color
  }
  else if (temp <= 25)
  {
    color = 0xff9933; //orange color
  }
  else {
    color = 0xff0000; // red color
  }

  for (int i = 0; i < 10; i++) {
    CircuitPlayground.setPixelColor(i, color);
  }
}

BLYNK_WRITE(V8) {
  int z = param[2].asFloat();  //read device's Z value
  if (z < 0) // checks if the phone is turned
  {
    nightMode = true; //change to nightMode
  }
  else {
    nightMode = false; //change to dayMode
  }
}
